@@ -352,7 +352,7 @@ struct LogicalLineInfo {
     kind: LogicalLineKind,
     first_token_range: TextRange,
 
-    // The token's kind right before the newline ending the logical line.
+    // The kind of the last non-trivia token before the newline ending the logical line.
     last_token: TokenKind,
 
     // The end of the logical line including the newline.
@@ -549,7 +549,9 @@ impl<'a> Iterator for LinePreprocessor<'a> {
                 _ => {}
             }
 
-            last_token = token_kind;
+            if !token_kind.is_trivia() {
+                last_token = token_kind;
+            }
         }
 
         None
@@ -635,11 +637,26 @@ enum Follows {
     Other,
     Decorator,
     Def,
+    /// A function whose body is a dummy (...), if the ellipsis is on the same line as the def.
+    DummyDef,
     Import,
     FromImport,
     Docstring,
 }
 
+impl Follows {
+    // Allow a function/method to follow a function/method with a dummy body.
+    const fn follows_def_with_dummy_body(self) -> bool {
+        matches!(self, Follows::DummyDef)
+    }
+}
+
+impl Follows {
+    const fn is_any_def(self) -> bool {
+        matches!(self, Follows::Def | Follows::DummyDef)
+    }
+}
+
 impl Follows {
     const fn is_any_import(self) -> bool {
         matches!(self, Follows::Import | Follows::FromImport)
@@ -755,7 +772,11 @@ impl<'a> BlankLinesChecker<'a> {
                     if matches!(state.fn_status, Status::Outside) {
                         state.fn_status = Status::Inside(logical_line.indent_length);
                     }
-                    state.follows = Follows::Def;
+                    state.follows = if logical_line.last_token == TokenKind::Ellipsis {
+                        Follows::DummyDef
+                    } else {
+                        Follows::Def
+                    };
                 }
                 LogicalLineKind::Comment => {}
                 LogicalLineKind::Import => {
@@ -801,7 +822,8 @@ impl<'a> BlankLinesChecker<'a> {
             // Only applies to methods.
             && matches!(line.kind,  LogicalLineKind::Function | LogicalLineKind::Decorator)
             // Allow groups of one-liners.
-            && !(matches!(state.follows, Follows::Def) && !matches!(line.last_token, TokenKind::Colon))
+            && !(state.follows.is_any_def() && line.last_token != TokenKind::Colon)
+            && !state.follows.follows_def_with_dummy_body()
             && matches!(state.class_status, Status::Inside(_))
             // The class/parent method's docstring can directly precede the def.
             // Allow following a decorator (if there is an error it will be triggered on the first decorator).
@@ -852,7 +874,8 @@ impl<'a> BlankLinesChecker<'a> {
             // Allow following a decorator (if there is an error it will be triggered on the first decorator).
             && !matches!(state.follows, Follows::Decorator)
             // Allow groups of one-liners.
-            && !(matches!(state.follows, Follows::Def) && !matches!(line.last_token, TokenKind::Colon))
+            && !(state.follows.is_any_def() && line.last_token != TokenKind::Colon)
+            && !(state.follows.follows_def_with_dummy_body() && line.preceding_blank_lines == 0)
             // Only trigger on non-indented classes and functions (for example functions within an if are ignored)
             && line.indent_length == 0
             // Only apply to functions or classes.
@@ -1018,7 +1041,8 @@ impl<'a> BlankLinesChecker<'a> {
             // Do not trigger when the def/class follows an "indenting token" (if/while/etc...).
             && prev_indent_length.is_some_and(|prev_indent_length| prev_indent_length >= line.indent_length)
             // Allow groups of one-liners.
-            && !(matches!(state.follows, Follows::Def) && line.last_token != TokenKind::Colon)
+            && !(state.follows.is_any_def() && line.last_token != TokenKind::Colon)
+            && !state.follows.follows_def_with_dummy_body()
             // Blank lines in stub files are only used for grouping. Don't enforce blank lines.
             && !self.source_type.is_stub()
         {
