@@ -212,16 +212,11 @@ fn clean_params_tuple<'a>(right: &Expr, locator: &Locator<'a>) -> Cow<'a, str> {
 fn clean_params_dictionary(right: &Expr, locator: &Locator, stylist: &Stylist) -> Option<String> {
     let is_multi_line = locator.contains_line_break(right.range());
     let mut contents = String::new();
-    if let Expr::Dict(ast::ExprDict {
-        keys,
-        values,
-        range: _,
-    }) = &right
-    {
+    if let Expr::Dict(ast::ExprDict { items, range: _ }) = &right {
         let mut arguments: Vec<String> = vec![];
         let mut seen: Vec<&str> = vec![];
         let mut indent = None;
-        for (key, value) in keys.iter().zip(values.iter()) {
+        for ast::DictItem { key, value } in items {
             match key {
                 Some(key) => {
                     if let Expr::StringLiteral(ast::ExprStringLiteral {
