@@ -1,6 +1,7 @@
 mod cancel;
 mod did_change;
 mod did_change_configuration;
+mod did_change_watched_files;
 mod did_change_workspace;
 mod did_close;
 mod did_open;
@@ -9,6 +10,7 @@ use super::traits::{NotificationHandler, SyncNotificationHandler};
 pub(super) use cancel::Cancel;
 pub(super) use did_change::DidChange;
 pub(super) use did_change_configuration::DidChangeConfiguration;
+pub(super) use did_change_watched_files::DidChangeWatchedFiles;
 pub(super) use did_change_workspace::DidChangeWorkspace;
 pub(super) use did_close::DidClose;
 pub(super) use did_open::DidOpen;
